	 TITLE '***** DSOPEN -- OPEN DATASET *****'
*START OF SPECIFICATIONS*************************************
* MODULE NAME = DSOPEN					    *
* DESCRIPTIVE NAME = DSOPEN COPY CODE INTERFACE		    *
* COPYRIGHT = 5719-LM7 IBM CORP. 1981			    *
*	      5719-XX4 IBM CORP. 1981			    *
*	      5740-LM4 IBM CORP. 1981			    *
*	      REFER TO INSTRUCTIONS ON COPYRIGHT NOTICE     *
*	      FORM NO. G120-2083			    *
* CHANGE ACTIVITY = SEE BELOW				    *
*   VERSION = 03.0 MODIFIED				    *
*	    = 03.1 SEE CHANGES BELOW			    *
* FUNCTION = DATA SET OPEN COPY CODE MODULE		    *
* MODULE TYPE = COPY CODE				    *
* MACROS = NONE						    *
*END OF SPECIFICATIONS***************************************
*
*  CHANGE HISTORY
*
*  V3M1     JWW  11/17/80 - MODULE MODIFIED TO ADD FUNCTION
*			  -   AND KEEP STORAGE REQUIREMENTS SAME.
*			  - SEARCH VDE CHAIN INSTEAD OF DDB CHAIN FOR
*			  -  DIRECTORY DEVICES
*			  - PREVIOUS CHANGE FLAGS REMOVED(REWRITTEN)
*			  - EXTRANEOUS COMMENTS REMOVED.
*			  - SUPPORT $DSBLIB EXIT(VOL. NOT INITIALIZED)
*			  -   EXIT TO $DSBLIB,$DSBVOL,$DSNFND IF
*			  -   NOT INIT (IN THAT ORDER).
*			  -   (ONLY IF NOT $$...... DSNAME)
*			  - DISALLOW ACCESS TO VTOC(VOLUME DIRECTORY)
*			  -   VDE NAME.
*			  - READ DCE EVEN IF $$...... DSNAME
*			  - FIX UP FIELDS $DSCBLAD/PSZ/ENT/RLD FOR PGMS
*			  - SET FIELD $DSCBPGS FOR PGM BYTE SIZE
*			  - USE FIELD $DSCBNX2 FOR NOTE
*
*  @PT00063 JWW  04/14//1 - DONT SEARCH DIRECTORY IF EMPTY
*
*  NOTE - NOTE - NOTE - NOTE
*
*  IF COPY CODE MODULES DDODEFEQ,DSCBEQU,DDBEQU CHANGE
*  THEN DSOPEN MUST ALSO CHANGE BECAUSE ALL OR PARTS OF THESE
*  MODULES ARE USED... OK?
*
*  END OF NOTE
*
***************************************************************
***************************************************************
*	 ROOT DSOPEN
***************************************************************
***************************************************************
*
*  SUBROUTINE DSOPEN
*
*  LOCATE DATASET ON DATA MANAGEMENT DEVICE
*
*  CALLING SEQUENCE:
*
*      CALL   DSOPEN,DS#
*
*  INPUT:
*
*    DATASET CONTROL BLOCK DS# CONTAINING THE FOLLOWING VALUES:
*
*      $DSCBNAM = DATASET NAME
*      $DSCBVOL = VOLUME NAME (OR BLANKS)
*
*    DISKBUFR = 256 BYTE DISK INPUT BUFFER DEFINED BY CALLING PROGRAM
*
*    ERROR RETURN LABELS -- THESE LOCATIONS MUST BE INITIALIZED
*    APPROPRIATELY IN THE CALLING PROGRAM.  IF A LOCATION IS SET TO
*    ZERO, DSOPEN WILL RETURN TO THE CALLING PROGRAM WHEN THE
*    ASSOCIATED ERROR OCCURS.  (LABELS ARE DEFINED IN THE DATA AREA.)
*
*
*
*    DIRECTORY CONTROL ENTRY AND MEMBER ENTRY RETURN LABEL -- THIS
*    LOCATION MUST BE INITIALIZED IN THE CALLING PROGRAM.  IF IT IS NOT
*    ZERO, THE DIRECTORY CONTROL ENTRY AND THE FIRST MEMBER ENTRY WILL
*    BE STORED AT THE SPECIFIED LOCATION.  (LABEL IS DEFINED IN THE
*    DATA AREA.)
*
*
*  OUTPUT:
*
*    DATASET CONTROL BLOCK DS# CONTAINING THE FOLLOWING VALUES:
*
*      $DSCBVOL = VOLUME NAME OF IPL VOLUME (IF BLANKS SPECIFIED)
*      $DSCBORN = ABSOLUTE RECORD NUMBER OF DATASET ORIGIN
*      $DSCBNEX = 1
*      $DSCBLNG = NUMBER OF RECORDS IN DATASET
*      $DSCBVDE = VOLUME DESCRIPTOR ENTRY POINTER (DEVICE DESCRIPTOR
*		  IF NO VOLUME DESCRIPTOR)
*      $DSCBEND = IF TYPE IS DATA AND END OF DATA SPECIFIED: END OF
*		  DATA (-1 = EMPTY)
*		  IF TYPE IS DATA AND END OF DATA NOT SPECIFIED: 0
*		  IF TYPE IS NOT DATA: $DSCBLNG
*      $DSCBRCL = LOGICAL RECORD LENGTH
*      $DSCBEDB = NEXT AVAILABLE LOGICAL RECORD
*      $DSCBFLG = END OF DATA, FIXED HEAD, READ VERIFY FLAGS
*      $DSCBPGS = PGM SIZE IN BYTES
*      $DSCBLAD = PGM LOAD ADDRESS
*      $DSCBENT = PGM ENTRY POINT(PGM)
*      $DSCBRLD = PGM RLD COUNT
*
*    DATASET CONTROL BLOCK DS$ CONTAINING THE FOLLOWING VALUES:
*
*      $DSCBORN = ABSOLUTE RECORD NUMBER OF VOLUME ORIGIN
*      $DSCBNEX = 1
*      $DSCBLNG = NUMBER OF RECORDS IN VOLUME
*      $DSCBVDE = VOLUME DESCRIPTOR ENTRY POINTER (DEVICE DESCRIPTOR
*		  IF NO VOLUME DESCRIPTOR)
*      $DSCBEND = NUMBER OF RECORDS IN VOLUME
*      $DSCBFLG = END OF DATA, FIXED HEAD, READ VERIFY FLAGS
*
*    #1 = DIRECTORY RECORD NUMBER CONTAINING MEMBER ENTRY
*    #2 = DISPLACEMENT WITHIN DIRECTORY RECORD TO MEMBER ENTRY
*
*    DIRECTORY CONTROL ENTRY AND FIRST MEMBER ENTRY STORED AT $DSDCEA
*    (IF NON-ZERO)
*
*    EXIT VIA RETURN OR TO ERROR EXIT AS SPECIFIED.  IF ERRORS ARE
*    ENCOUNTERED, THE OUTPUT AREAS ARE UNDEFINED, EXCEPT AS NOTED.
*
	 SUBROUT DSOPEN,DS#
*
*  LOCATE VOLUME
*
	 MOVE  $DIRVDE,0,4	   $DIRVDE,$DCEVTOC,$VOLINIT,$PERF=0
	 MOVE  #1,DS#		   #1 = DATASET CONTROL BLOCK ADDRESS
	 IF    (#1,EQ,+DS$),OR,(($DSCBVOL,#1),EQ,0),GOTO,$DSO1
	 IF    (($DSCBVOL,#1),EQ,+X'4040')  IF VOL. NAME NOT SPECIFIED
	   MOVE  #2,$IPLVOL,FKEY=0 #2 = IPL VOLUME NAME POINTER
	   MOVE  ($DSCBVOL,#1),(0,#2),(6,BYTES),FKEY=0 FETCH IPL VOL
	 ENDIF
	 CALL  GETVOL,DS#	   LOCATE SPECIFIED VOLUME
*
*  OPEN DATASET -- TAPE VOLUME SPECIFIED
*
	 MOVE  #1,DS#		   #1 = DATA SET CONTROL BLOCK ADDRESS
	 MOVE  #2,($DSCBVDE,#1)    #2 = VOLUME DESCRIPTOR POINTER
	 MOVE  #2,($VOLDDB,#2),FKEY=0 #2 = DEVICE DATA BLOCK POINTER
	 MOVE  #2,($DDBTYPE,#2),FKEY=0 #2 = DEVICE IDENTIFICATION
	 IF    (#2,EQ,+$D4969)	   IF TAPE DEVICE
	   MOVE  $CNTL+10,#1	   STORE DSCB ADDR IN CONTROL INSTR.
$CNTL	   CONTROL *,OPEN	   OPEN TAPE DATASET
	   IF	 (($DSCBECB,#1),NE,-1) IF ERROR
	     IF    (($DSCBECB,#1),EQ,76),AND,($DSNFND,NE,0) DS NOT FND
$DSNFND$       GOTO  ($DSNFND)	   RETURN TO NOT FOUND EXIT
	     ENDIF
	     IF    ($DSIOERR,NE,0) IF I/O ERROR EXIT SPECIFIED
$DSIOER$       GOTO  ($DSIOERR)    RETURN TO I/O ERROR EXIT
	     ENDIF
	   ENDIF
*
*  OPEN DATASET -- DISK VOLUME SPECIFIED
*
	 ELSE
	   CALL  FINDNAME,DS#	   OPEN SPECIFIED DATASET
*
*  CHECK IF SYSTEM FLAG NOT SET
*
	   IF	 ($DDEVDE,EQ,0)
*
*  IF A DISKETTE DIRECTORY DEVICE AND A MULTI-VOL DISKETTE
*   GIVE AN ERROR
*
	     IF  ($DIRVDE,EQ,+X'2800'),AND,($DCEVTOC,EQ,+X'1800'),     X
	       GOTO,$DSO1
*
*  IF A DISK DIRECTORY DEVICE, GIVE AN ERROR
*
	     IF  ($DIRVDE,EQ,+X'0800')
$DSO1	       IF ($DSBVOL,NE,0)   IF NO ERROR EXIT
$DSBVOL$	 GOTO ($DSBVOL)    RETURN TO VOLUME ERROR EXIT
	       ENDIF
	     ENDIF
	   ENDIF
	 ENDIF
	 RETURN
***************************************************************
***************************************************************
*	 END OF ROOT DSOPEN
***************************************************************
***************************************************************
	 EJECT
*//
***************************************************************
***************************************************************
*	 GETVOL
***************************************************************
***************************************************************
*
*  SUBROUTINE GETVOL
*								      *
***********************************************************************
***********************************************************************
*								      *
*  LOCATE DATA MANAGEMENT VOLUME
*
*  CALLING SEQUENCE:
*
*	 CALL  GETVOL,DS##
*
*  INPUT:
*
*    DATASET CONTROL BLOCK DS## CONTAINING THE FOLLOWING VALUES:
*
*      $DSCBVOL = VOLUME NAME
*
*  OUTPUT:
*
*    VALID VOLUME NAME AND VOLUME ONLINE
*
*      DATASET CONTROL BLOCK DS## CONTAINING THE FOLLOWING VALUES:
*
*	 $DSCBVDE = VOLUME DESCRIPTOR ADDRESS (DEVICE DESCRIPTOR
*		    ADDRESS IF NO VOLUME DESCRIPTOR)
*
*      DATASET CONTROL BLOCK DS$ CONTAINING THE FOLLOWING VALUES
*
*	 $DSCBORN = ABSOLUTE RECORD NUMBER OF VOLUME ORIGIN
*	 $DSCBNEX = 1
*	 $DSCBLNG = NUMBER OF RECORDS IN VOLUME
*	 $DSCBVDE = VOLUME DESCRIPTOR ENTRY POINTER (DEVICE DESCRIPTOR
*		    IF NO VOLUME DESCRIPTOR
*	 $DSCBEND = NUMBER OF RECORDS IN VOLUME
*	 $DSCBFLG = END OF DATA, FIXED HEAD, READ VERIFY FLAGS
*
*      EXIT VIA RETURN
*
*    VALID VOLUME NAME AND VOLUME UNUSABLE
*
*      DATASET CONTROL BLOCK DS## CONTAINING THE FOLLOWING VALUES:
*
*	 $DSCBVDE = VOLUME DESCRIPTOR ADDRESS
*
*      EXIT TO $DSBVOL (RETURN IF $DSBVOL NOT SPECIFIED)
*
*    INVALID VOLUME NAME
*
*      DATASET CONTROL BLOCK DS## CONTAINING THE FOLLOWING VALUES:
*
*	 $DSCBVDE = 0
*
*      EXIT TO $DSBVOL (RETURN IF $DSBVOL NOT SPECIFIED)
*
*    #1, #2 ARE USED
*
	 SUBROUT GETVOL,DS##
	 MOVE  $DSNFSAV,$DSNFND,5 SAVE ERROR EXITS
	 MOVEA $ERRSAV,$PASSNFN    SET UP TEMP ERROR EXITS
	 MOVE  $DSNFND,0,5,P2=$ERRSAV  TEMPORARY ERROR EXITS
*
*  SEARCH FOR VOLUME DESCRIPTOR
*
	 DO    2,TIMES,INDEX=$PERF MAKE TWO PASSES THRU VDE
	   MOVE #2,$DMVOL,FKEY=0   #2 = VDE POINTER
	   MOVE $DSOTMP2,($VOLNAME,#2),(6,BYTES),FKEY=0
	   DO  WHILE,(#2,NE,0)	   SEARCH VOLUME DESCRIPTOR CHAIN
	     MOVE #1,DS##
*
*    IF FOUND, COMPLETE DS$
*
	     MOVE DS$+$DSCBVDE,#2  SAVE #2
	     IF  ($PERF,EQ,1)
	       IF ($DSOTMP2,NE,($DSCBVOL,#1),6),OR,		       X
	       ($DSOTMP2,EQ,0),GOTO,$PASSNFN
	     ELSE
*
*    SEARCH VOLUME DIRECTORIES
*
	       MOVE DS$+$DSCBNAM,($DSCBVOL,#1),(6,BYTES) STORE VOL NAME
	       MOVE DS$+$DSCBNME,C'  ' CLEAR REST OF NAME FIELD
	       MOVE $DSOTMP1,($VOLFLAG,#2),FKEY=0 GET VDE FLAGS
	       AND $DSOTMP1,+X'0800' CHECK FOR DIRECTORY DEVICE
	       MOVE $DSOTMP2,($VOLNAME,#2),FKEY=0
	       IF ($DSOTMP1,EQ,0),OR,($DSOTMP2,EQ,0),GOTO,$PASSNFN
	     ENDIF
	     MOVE DS$+$DSCBORN,($VOLORG,#2),2,FKEY=0 VOLUME ORIGIN
	     MOVE DS$+$DSCBNEX,1,DWORD SET NEXT RECORD POINTER
	     MOVE DS$+$DSCBLNG,($VOLSIZE,#2),2,FKEY=0	 VOLUME SIZE
	     MOVE DS$+$DSCBEND,($VOLSIZE,#2),2,FKEY=0 VOL END OF DATA
*  CLEAR FLAG WORD (EOD, FIXED HEAD, WRITE VERIFY)
	     MOVE DS$+$DSCBFLG,0	CLEAR FLAGS WORD
	     MOVE $DSOTMP1,($VOLFLAG,#2),FKEY=0 GET FLAGS
	     AND $DSOTMP1,+X'2800',RESULT=$DIRVDE SAVE FLAGS
	     IF    ($PERF,EQ,1)
	       AND $DSOTMP1,+$VOLFH TEST FIXED HEAD VOLUME FLAG
	       SHIFTL $DSOTMP1,3
	       IOR DS$+$DSCBFLG,$DSOTMP1 SET FIXED HEAD FLAG
	     ELSE
*
*    SEARCH VOLUME DIRECTORY
*
	       CALL FINDNAME,(DS$) SEARCH VOLUME DIRECTORY FOR NAME
	       AND  $DIRVDE,X'F7FF' CLEAR DIRECTORY FLAG
	     ENDIF
*
*  VOLUME FOUND
*
$VNFOUND     EQU *		   VOLUME NAME FOUND
	     MOVE $DSNFND,$DSNFSAV,5 RESTORE ERROR EXITS
	     MOVE #2,DS$+$DSCBVDE RESTORE #@ - VDE POINTER
	     MOVE #1,DS##	   #1 = DATASET CONTROL BLOCK ADDRESS
	     MOVE ($DSCBVDE,#1),#2 GET VOLUME DESCRIPTOR POINTER
	     MOVE #2,($VOLDDB,#2),FKEY=0 #2 = DDB
	     MOVE #2,($DDBFLAG,#2),FKEY=0 #2 = DDB FLAG
	     AND #2,+$DDBUSE,RESULT=$DSOTMP1
	     IF ($DSOTMP1,NE,0),GOTO,$VNNTUSE IF UNUSABLE, ERROR
	     RETURN
$PASSNFN     EQU   *
	     MOVE  #2,DS$+$DSCBVDE RESTORE INDEX REGISTER #2
	     MOVE #2,($VOLNEXT,#2),FKEY=0 #2 = NEXT VDE
	     MOVE $DSOTMP2,($VOLNAME,#2),(6,BYTES),FKEY=0 $DSOTMP2 =
*				     VOLUME DESCRIPTOR NAME FIELD
	   ENDDO
	 ENDDO
*
*  VOLUME NOT FOUND
*
	 MOVE  $DSNFND,$DSNFSAV,5  RESTORE ERROR EXITS
	 MOVE  #1,DS##		   #1 = DATASET CONTROL BLOCK ADDRESS
	 MOVE  ($DSCBVDE,#1),0	   RESET VOLUME DESCRIPTOR POINTER
$VNNTUSE EQU   *		   VOLUME UNUSABLE		D105065
	 IF    ($DSBVOL,NE,0),GOTO,$DSBVOL$ ERROR EXIT
	 RETURN
***************************************************************
***************************************************************
*	 END OF GETVOL
***************************************************************
***************************************************************
	 EJECT
*//
***************************************************************
***************************************************************
*	 FINDNAM
***************************************************************
***************************************************************
*
*
*  SUBROUTINE FINDNAME
*
*  LOCATE NAME IN DIRECTORY
*
*  CALLING SEQUENCE:
*
*	 CALL  FINDNAME,DS###
*
*  INPUT:
*
*    DATASET CONTROL BLOCK DS### CONTAINING THE FOLLOWING VALUES:
*
*      $DSCBNAM = MEMBER NAME
*
*    DATASET CONTROL BLOCK DS$ CONTAINING THE FOLLOWING VALUES:
*
*      $DSCBORN = DEVICE ORIGIN OR VOLUME ORIGIN (ABSOLUTE RECORD
*		  OF VOLUME OR DATASET DIRECTORY)
*      $DSCBNEX = 1
*      $DSCBLNG = NUMBER OF RECORDS ON DEVICE OR IN VOLUME
*      $DSCBVDE = VOLUME DESCRIPTOR ENTRY POINTER (DEVICE DESCRIPTOR
*		  IF NO VOLUME DESCRIPTOR)
*      $DSCBEND = NUMBER OF RECORDS ON DEVICE OR IN VOLUME
*      $DSCBFLG = END OF DATA, FIXED HEAD, READ VERIFY FLAGS
*
*    DISKBUFR -- 256 BYTE DISK INPUT BUFFER
*
*  OUTPUT:
*
*    NAME FOUND
*
*      DATASET CONTROL BLOCK DS### CONTAINING THE FOLLOWING VALUES:
*
*	 $DSCBORN = ABSOLUTE RECORD NUMBER OF VOLUME OR DATASET ORIGIN
*	 $DSCBNEX = 1
*	 $DSCBLNG = NUMBER OF RECORDS IN VOLUME OR DATASET
*	 $DSCBVDE = VOLUME DESCRIPTOR ENTRY POINTER (DEVICE DESCRIPTOR
*		    IF NO VOLUME DESCRIPTOR)
*	 $DSCBEND = IF TYPE IS DATA AND END OF DATA SPECIFIED: END OF
*		    DATA (-1 = EMPTY)
*		    IF TYPE IS DATA AND END OF DATA NOT SPECIFIED: 0
*		    IF TYPE IS NOT DATA: $DSCBLEN
*	 $DSCBRCL = LOGICAL RECORD LENGTH
*	 $DSCBEDB = NEXT AVAILABLE LOGICAL RECORD
*	 $DSCBFLG = END OF DATA, FIXED HEAD, READ VERIFY FLAGS
*
*      #1 = DIRECTORY RECORD NUMBER CONTAINING MEMBER ENTRY
*      #2 = DISPLACEMENT WITHIN DISKBUFR TO MEMBER ENTRY
*
*      EXIT VIA RETURN
*
*    DSNAME = $$, $$EDXVOL, OR $$EDXLIB
*
*      $$EXIT NOT 0
*
*	 RETURN TO $$EXIT
*
*      $$EXIT = 0
*
*	 DATASET CONTROL BLOCK DS### CONTAINING THE FOLLOWING VALUES:
*
*	   $DSCBORN = DEVICE ORIGIN OR VOLUME ORIGIN (ABSOLUTE RECORD
*		      NUMBER OF VOLUME DIRECTORY OR DATASET DIRECTORY)
*	   $DSCBNEX = 1
*	   $DSCBLNG = NUMBER OF RECORDS ON DEVICE OR IN VOLUME
*	   $DSCBVDE = VOLUME DESCRIPTOR ENTRY POINTER (DEVICE
*		      DESCRIPTOR IF NO VOLUME DESCRIPTOR)
*	   $DSCBEND = NUMBER OF RECORDS ON DEVICE OR IN VOLUME
*	   $DSCBFLG = END OF DATA, FIXED HEAD, READ VERIFY FLAGS
*
*    NAME NOT FOUND
*
*      EXIT TO $DSNFND, IF SPECIFIED
*
*    DIRECTORY NOT INITIALIZED
*
*      EXIT TO $DSBLIB, IF SPECIFIED
*	ELSE EXIT TO $DSBVOL, IF SPECIFIED
*	 ELSE EXIT TO $DSNFND, IF SPECIFIED
*
*    DISK I/O ERROR
*
*      EXIT TO $DSIOERR, IF SPECIFIED
*
	 SUBROUT FINDNAME,DS###
*
*  PROCESS $$, $$EDXLIB, $$EDXVOL
*
	 READ  DS$,DISKBUFR,1,ERROR=$FNIOERR READ DCE
	 IF    (DISKBUFR+$$FPDINT,NE,$EDXEDX,6) DIR NOT INIT
	   MOVE $VOLINIT,1	   SET VOLINIT FLAG = NOT INIT
	 ELSE
	   MOVE $VOLINIT,0	   SET VOLINIT FLAG = INIT
	   AND DISKBUFR+$$FPDFLG,+X'1800',RESULT=$DCEVTOC
	   AND DISKBUFR+$$FPDFLG,+X'4000',RESULT=$WRITVER  GET VERIFY
	 ENDIF
	 MOVE  #1,DS###		   #1 = DATASET CONTROL BLOCK ADDRESS
	 IF    (#1,EQ,+DS$)	   CHECK IF VOLUME LOOKUP
	   IF  ($DIRVDE,EQ,+X'2800'),AND,($DCEVTOC,NE,+X'1800'),       X
	       GOTO,$FNNO
	 ENDIF
*
*  IF OPENING TO DISKETTE AND IT IS NOT A MULTI-VOLUME TYPE
*   CLEAR DIRECTORY BIT IN VDE
*   (NEVER IMPLEMENTED)
*
*	 IF	(#1,EQ,+DS$),AND,($DIRVDE,EQ,X'2800')
*	   IF	($DCEVTOC,NE,X'1800'),OR,($VOLINIT,EQ,1)
*  VDE ADDRESS IS IN #2
*	     MOVE $DSOTMP1,($VOLFLAG,#2),FKEY=0 GET VOL FLAGS
*	     AND $DSOTMP1,X'F7FF' CLEAR DIRECTORY BIT
*	     MOVE ($VOLFLAG,#2),$DSOTMP1,TKEY=0 AND PUT INTO VDE
*	   ENDIF
*	 ENDIF
	 IF    ($DSDCEA,NE,0)	   CHECK DCE REQUIREMENT
	   MOVE  #2,$DSDCEA	   #2 = SPECIFIED ADDRESS
	   MOVE  (0,#2),DISKBUFR,(32,BYTES) STORE DCE ENTRY
	 ENDIF
	 MOVE  ($DSCBFLG,#1),0	   CLEAR FLAGS WORD
	 MOVE  #2,($DSCBVDE,#1)    #2 = VOLUME DESCRIPTOR ADDRESS
	 IF  (($DSCBNAM,#1),EQ,$$VOL,8)
	   MOVE DS$+$DSCBORN,1,DWORD
	   GOTO $$VOLLIB
	 ENDIF
	 IF    (($DSCBNAM,#1),EQ,$$$NAME,8),OR,			       $
	       (($DSCBNAM,#1),EQ,$$LIB,8)
$$VOLLIB   EQU	 *
	   IF	 ($$EXIT,NE,0)	   IF $$EXIT SPECIFIED
	     GOTO  ($$EXIT)	   RETURN TO $$EXIT
	   ENDIF
	   MOVE  DS$+$DSCBNEX,1,DWORD  POINT TO REC. 1
	   MOVE  ($DSCBORN,#1),DS$+$DSCBORN,(3,DWORD) $DSCBORN,NEX,LNG
	   MOVE  ($DSCBEND,#1),DS$+$DSCBEND,DWORD MOVE $DSCBEND
	   AND	 DS$+$DSCBFLG,+X'A000',RESULT=$DSOTMP1 GET EOD/FIXHD
	   IOR	 ($DSCBFLG,#1),$DSOTMP1 SET DATASET FLAGS
	   IOR	 ($DSCBFLG,#1),$WRITVER SET WRITE VERIFY FLAG
*
*  BUILD MEMBER ENTRY IN DISKBUFR FOR $$, $$EDXLIB, OR $$EDXVOL
*
	   MOVE  DISKBUFR,0,(32,BYTES) CLEAR DISKBUFR MEMBER ENTRY
	   MOVE  DISKBUFR,($DSCBNAM,#1),(8,BYTES) STORE NAME
	   SUBTRACT DS$+$DSCBLNG,1,PREC=DSD,RESULT=DISKBUFR+$$FPME
	   MOVE  DISKBUFR+$$FPMT,+$$FPMDAT STORE TYPE INDICATOR
	   MOVE  #2,0		   #2 = MEMBER ENTRY DISPLACEMENT
	   IF	 ($VOLINIT,NE,0),AND,($DSBLIB,NE,0),GOTO,$VOLUNIN
	   GOTO  $FNRTRN	   RETURN IF $$...... DATASET
	 ENDIF
	 IF	 ($VOLINIT,NE,0)  IF VOL. NOT INIT
*
* IF $DSBLIB IS USED - GO TO IT; IF NOT, THEN GO TO
*	  THE $DSBVOL EXIT, ELSE GO TO $DSNFND EXIT.
*
	   IF	 ($DSBLIB,NE,0)
$VOLUNIN     EQU  *		   VOL NOT INITIALIZED
	     GOTO ($DSBLIB)	   RETURN TO LIB NOT INIT EXIT
	   ENDIF
	   IF	 ($DSBVOL,NE,0),GOTO,$DSBVOL$
	   GOTO  $FNNO
	 ENDIF
	 MOVE  $FNCNT,DISKBUFR+$$FPDM $FNCNT = NUMBER OF MEMBERS
	 MOVE  #2,32		   #2 = MEMBER ENTRY DISPLACEMENT
*
*  SEARCH DIRECTORY FOR NAME
*
	 DO    WHILE,($FNCNT,NE,0) SEARCH DIRECTORY	       @PT00063
	   IF	 (($DSCBNAM,#1),EQ,(DISKBUFR+$$FPMN,#2),8),GOTO,$FNYES
*				   IF NAME FOUND, END SEARCH
	   ADD	 #2,32		   ADVANCE MEMBER ENTRY POINTER
	   IF	 (#2,GE,256)	   IF DIRECTORY RECORD EXHAUSTED
	     READ  DS$,DISKBUFR,1,ERROR=$FNIOERR READ DIRECTORY RECORD
	     MOVE  #2,0		   RESET MEMBER ENTRY POINTER
	   ENDIF
	   SUBTRACT $FNCNT,1	   DECREMENT MEMBER COUNT      @PT0063
	 ENDDO
*
*  UNSUCCESSFUL SEARCH
*
$FNNO	 EQU   *
	 IF    ($DSNFND,NE,0),GOTO,$DSNFND$
	 RETURN
$FNIOERR EQU   *		     I/O ERROR READING DIRECTORY
	 IF    ($DSIOERR,NE,0),GOTO,$DSIOER$
$FNRTRN  RETURN
*
*  SUCCESSFUL SEARCH
*
$FNYES	 EQU   *
	 ADD   (DISKBUFR+$$FPMS,#2),DS$+$DSCBORN,RESULT=($DSCBORN,#1), $
	       PREC=DDD		   COMPUTE DATASET ORIGIN
	 MOVE  ($DSCBNEX,#1),1,DWORD STORE NEXT RECORD POINTER
	 SUBTRACT (DISKBUFR+$$FPME,#2),(DISKBUFR+$$FPMS,#2),	       $
	       RESULT=($DSCBLNG,#1),PREC=DDD
	 ADD   ($DSCBLNG,#1),1,PREC=DS COMPUTE DATASET LENGTH
*
*  $DSCBLAD/RCL <== $$FPMA/L
*  $DSCBPSZ/END <== $$FPMB/F
*  $DSCBENT/EDB <== $$FPMP/D
*  $DSCBRLD/... <== $$FPMR/
*
	 MOVE  ($DSCBRCL,#1),(DISKBUFR+$$FPML,#2),5
*  $DSCBPGS	<== $$FPMB
	 MOVE  ($DSCBPGS,#1),(DISKBUFR+$$FPMB,#2)
*
*  COMPUTE LOGICAL END OF FILE
*
	 IF    ((DISKBUFR+$$FPMT,#2),EQ,+$$FPMDAT) IF TYPE IS DATA
	   IOR	 ($DSCBFLG,#1),+$DSCBEOD SET END OF DATA FLAG
	   IF	 ((DISKBUFR+$$FPMD,#2),NE,0) IF PARTIAL RECORD
	     ADD   ($DSCBEND,#1),1,PREC=DS END OF DATA = END OF DATA
*				   + 1
	   ELSE
	     IF    ((DISKBUFR+$$FPMF,#2),EQ,$DWRD0,DWORD) IF EMPTY
*				   DATA SET
	       AND   (DISKBUFR+$$FPMFLG,#2),+$DMEEOD,RESULT=$DSOTMP1
*
	       IF    ($DSOTMP1,NE,0)   IF END OF DATA ASSIGNED
		 MOVE  ($DSCBEND,#1),+X'FFFF',DWORD END OF DATA = -1
*
	       ELSE
		 AND   ($DSCBFLG,#1),+X'7FFF' RESET EOD FLAG
*
	       ENDIF
	     ENDIF
	   ENDIF
	 ELSE			   TYPE IS NOT DATA
	   MOVE  ($DSCBEND,#1),($DSCBLNG,#1),DWORD END OF DATA = LENGTH
	 ENDIF
*
*  STORE LOGICAL END OF DATA INFORMATION, COMPUTE DIRECTORY RECORD
*  NUMBER
*
*				   LOGICAL RECORD
	 IF    (#1,EQ,+DS$)	   IF OPENING VOLUME
	   AND	 (DISKBUFR+$$FPMFLG,#2),+$DMEFH,RESULT=$DSOTMP1 TEST
*				   FIXED HEAD VOLUME FLAG
	   IF	 ($DSOTMP1,NE,0)
	     IOR   DS$+$DSCBFLG,+$DSCBFH SET FIXED HEAD VOLUME FLAG
	     MOVE  DS$+$DSCBORN,1,DWORD SET ORIGIN TO ZERO
	   ENDIF
	 ELSE			   OPENING DATASET
	   AND	 DS$+$DSCBFLG,+X'A000',RESULT=$DSOTMP1 GET EOD/FIXHD
	   IOR	 ($DSCBFLG,#1),$DSOTMP1 SET EOD/FIXHD
	   IOR	 ($DSCBFLG,#1),$WRITVER SET WRITE VERIFY FLAG
	 ENDIF
	 SUBTRACT DS$+$DSCBNX2,1,RESULT=#1  #1 = DIR. REC. NO.
	 RETURN
***************************************************************
***************************************************************
*	 END OF FINDNAM
***************************************************************
***************************************************************
	 EJECT
*//
***************************************************************
***************************************************************
*	 ENQDEQV
***************************************************************
***************************************************************
*
*
*  SUBROUTINE ENQVOL
*
*  OBTAIN EXCLUSIVE USE OF VOLUME/DATASET DIRECTORY
*
*  CALLING SEQUENCE:
*
*	 CALL  ENQVOL,ENQDSCB,BUSYADDR
*
*  INPUT:
*
*    ENQDSCB = ADDRESS OF DSCB CONTAINING VOLUME DESCRIPTOR POINTER
*	       FOR VOLUME TO BE ENQUEUED
*
*    BUSYADDR = RETURN ADDRESS IF VOLUME IN USE (OPTIONAL)
*
*  OUTPUT:
*
*    VOLUME AVAILABLE
*
*      CALLING TASK RECEIVES EXCLUSIVE USE OF DIRECTORY
*
*    VOLUME NOT AVAILABLE
*
*      RETURN TO BUSYADDR (WAIT IF BUSYADDR NOT SPECIFIED)
*
	 SUBROUT ENQVOL,ENQDSCB,BUSYADDR
	 MOVE  $ESAV#1,#1	   SAVE INDEX REGISTER #1
	 MOVE  #1,ENQDSCB	   #1 = DATASET CONTROL BLOCK
	 MOVE  ENQVOL20,($DSCBVDE,#1) ENQVOL20 = VOLUME DESCRIPTOR
	 ADD   ENQVOL20,+$VOLQCB   ENQVOL20 = VOLUME RCB
	 MOVE  ENQVOL21,ENQVOL20   MOVE RESOURCE
	 TCBGET $ADS,$TCBADS	   TCBADS = $TCBADS
	 TCBPUT $DWRD0,$TCBADS	 $TCBADS = 0
	 IF	(BUSYADDR,EQ,0)     IF 0, WAIT FOREVER
	   ENQ	$DMMY,P1=ENQVOL21   ENQUEUE FOR RESOURCE
	 ELSE
	   ENQ	$DMMY,BUSY=$ENQBUSY,P1=ENQVOL20 ENQUEUE RCB
	 ENDIF
	 TCBPUT $ADS,$TCBADS	   $TCBADS = TCBADS
	 MOVE  #1,$ESAV#1	   RESTORE INDEX REGISTER #1
	 RETURN
$ENQBUSY EQU   *
	 TCBPUT $ADS,$TCBADS	   $TCBADS = TCBADS
	 MOVEA #1,0,P2=$ESAV#1	   RESTORE INDEX REGISTER #1
	 GOTO	(BUSYADDR)	   RETURN TO BUSY EXIT
*
*  SUBROUTINE DEQVOL
*
*  RELEASE EXCLUSIVE USE OF VOLUME/DATASET DIRECTORY
*
*  CALLING SEQUENCE:
*
*	 CALL  DEQVOL,DEQDSCB
*
*  INPUT:
*
*    DEQDSCB = ADDRESS OF DSCB CONTAINING VOLUME DESCRIPTOR POINTER
*	       FOR VOLUME TO BE DEQUEUED
*
*  OUTPUT:
*
*    CALLING TASK NO LONGER HAS EXCLUSIVE USE OF DIRECTORY
*
	 SUBROUT DEQVOL,DEQDSCB
	 MOVE  $DSAV#1,#1	   SAVE INDEX REGISTER #1
	 MOVE  #1,DEQDSCB	   #1 = DATASET CONTROL BLOCK
	 MOVE  DEQVOL20,($DSCBVDE,#1) DEQVOL20 = VOLUME DESCRIPTOR
	 ADD   DEQVOL20,+$VOLQCB   DEQVOL20 = VOLUME RESOURCE CONTROL
	 TCBGET $ADS,$TCBADS	   TCBADS = $TCBADS
	 TCBPUT $DWRD0,$TCBADS	 $TCBADS = 0
	 DEQ   $DMMY,P1=DEQVOL20   DEQUEUE RESOURCE
	 TCBPUT $ADS,$TCBADS	   $TCBADS = TCBADS
	 MOVEA #1,0,P2=$DSAV#1	   RESTORE INDEX REGISTER #1
	 RETURN
***************************************************************
***************************************************************
*	 END OF ENQDEQV
***************************************************************
***************************************************************
	 EJECT
*//
***************************************************************
***************************************************************
*	 DSODATA
***************************************************************
***************************************************************
*
* OPEN ERROR EXITS AND EXTERNAL INTERFACES
*  EXITS TAKEN: $DSNFND - VOLUME FOUND BUT DS NOT FOUND
*		$DSIOERR - I/O ERROR DURING DIRECTORY LOOKUP
*		$$EXIT	 - VOLUME FOUND BUT $$... DS SPECIFIED
*		$DSBLIB  - VOLUME FOUND BUT NOT INITIALIZED
*			   IF $DSBLIB = 0 GOTO $DSBVOL EXIT
*			   IF $DSBVOL = 0 GOTO $DSNFND EXIT
*		$DSBVOL  - VOLUME NOT FOUND
*
$DSNFND  DATA  A(*-*)		   DATASET NOT FOUND
$DSIOERR DATA  A(*-*)		   I/O ERROR DURING OPEN
$$EXIT	 DATA  A(*-*)		   DSN = $$,$$EDXLIB,$$EDXVOL
$DSBLIB  DATA  A(*-*)		   VOLUME NOT INITIALIZED
$DSBVOL  DATA  A(*-*)		   VOLUME NOT FOUND OR UNUSABLE
$DSDCEA  DATA  A(*-*)		   STORAGE AREA FOR DIRECTORY CONTROL
*				   ENTRY AND FIRST MEMBER ENTRY (0 = DO
*				   NOT STORE)
$DSBIODA DATA  A(*-*)		   INVALID DEVICE ADDRESS (NOT USED)
*				      RESERVED FOR FUTURE USE
$DSNVTOC DATA  A(*-*)		   NO VOLUME DIRECTORY (NOT USED)
*				      RESERVED FOR FUTURE USE
$DDEVDE  DATA  F'0'		   DDE OPEN  (SYSTEM USE ONLY)
*				   ^=0 = ALLOW OPEN TO DDE
*				   =0  = NO DDE OPEN ALLOWED
*					 DDE = SYSTEM VOLUME(DISK DIR.)
*					       OR MULTI-VOL DSKT VOL1
$DIRVDE  DATA  F'0'		   FLAGS
*				   X'0800' - DISK DIR VDE
*				   X'2800' - DISKETTE DIR VDE
$DCEVTOC DATA  F'0'		   FLAGS
*				   X'1000' - DISK VTOC DCE
*				   X'1800' - DISKETTE VTOC DCE
$PERF	 DATA  F'0'		   PASS 1/PASS 2 FLAG
*				   =1 = VOL FOUND AS PERF. VOL
*				   =2 = VOL NOT FOUND AS PERF. VOL
$VOLINIT DATA  F'0'		   =0 = VOLUME FOUND AND INITIALIZED
*				   =1 = VOLUME FOUND AND NOT INIT.
*
*  EQUATED CONSTANTS
*
$D4969	 EQU   X'3186'		   TAPE DEVICE IDENTIFICATION
$DDBUSE  EQU   X'4000'		   DEVICE DATA BLOCK -- USABLE
$DDBVDD  EQU   X'0400'		   DEVICE DATA BLOCK -- VOL DIR.
$VOLFH	 EQU   X'0400'		   VOLUME DESCRIPTOR -- FIXED HEAD
$DSCBEOD EQU   X'8000'		   DATASET CONTROL BLOCK -- END OF DATA
$DSCBRDV EQU   X'4000'		   DATASET CONTROL BLOCK -- WRITE VER.
$DSCBFH  EQU   X'2000'		   DATASET CONTROL BLOCK -- FIXED HEAD
$DMEEOD  EQU   X'8000'		   DIRECTORY MEMBER ENTRY - END OF DATA
$DMERDV  EQU   X'4000'		   DIRECTORY MEMBER ENTRY - WRITE VER.
$DMEFH	 EQU   X'2000'		   DIRECTORY MEMBER ENTRY -- FIXED HEAD
*
*  DATA AREA
*
$DMMY	 QCB   0		   DUMMY RESOURCE CONTROL BLOCK
$DWRD0	 EQU   $DMMY		   DOUBLEWORD ZERO CONSTANT
$DSOTMP1 EQU   $DMMY+4		   WORK AREA
$FNCNT	 EQU   $DMMY+4		   MEMBER COUNT
$WRITVER EQU   $DMMY+6		   WRITE VERIFY FLAG - SET W/DCE READ
$ADS	 EQU   $DMMY+8		   TCBADS SAVE AREA
$EDXEDX  DATA  CL6'EDXEDX'	   DIRECTORY INITIALIZATION STRING
$$$NAME  DATA  CL8'$$'		   DSNAME -- OPEN TO DEVICE
$$VOL	 DATA  CL8'$$EDXVOL'	   DSNAME -- OPEN TO DEVICE
$$LIB	 DATA  CL8'$$EDXLIB'	   DATASET NAME -- OPEN TO DIRECTORY
$DSOTMP2 DATA  3F'0'		   WORK AREA
$DSNFSAV DATA  5A(*-*)		   INTERNAL FINDNAME SAVE AREA
*
*  DATA SET CONTROL BLOCK
*
	 DSCB  DS#=DS$,DSNAME=$$
	 DATA  11F'0'					       @PT00249
***************************************************************
***************************************************************
*	 END OF DSODATA
***************************************************************
***************************************************************
	 EJECT
*//
***************************************************************
***************************************************************
*	 DDODEFEQ
***************************************************************
***************************************************************
*
*	 THIS IS THE SAME AS DDODEFEQ IN MACRO LIBRARY
*
* NUMBERS IN PARENTHESES (X) INDICATE LENGTH OF FIELD IN BYTES...
*
***********************************
*  DIRECTORY CONTROL ENTRY  (DCE) *
***********************************
*
$$DCE	 EQU   *-*	    BEGINNING OF DCE
$$FPD	 EQU   $$DCE	    START FIRST FIELD
$$FPDS	 EQU   $$FPD+0	    NO. RECORDS IN VOLUME OR DEVICE (4)
$$FPDE	 EQU   $$FPDS+4     NO. FREE RECORDS IN VOLUME, OR DEVICE (4)
$$FPDM	 EQU   $$FPDE+4     NO. MEMBER ENTRIES	(2)
$$FPDF	 EQU   $$FPDM+2     NO. FREE SPACE ENTRIES (2)
$$FPDD	 EQU   $$FPDF+2     NO. RECORDS IN DIRECTORY (2)
$$FPDB	 EQU   $$FPDD+2     NO. UNUSED BYTES IN DIRECTORY (2)
$$FPDINT EQU   $$FPDB+2     INITIALIZATION CHARACTERS (6)
$$FPDFLG EQU   $$FPDINT+6   FLAGS: (2)
*			    BIT 0 = RESERVED, MUST BE ZERO
*			    BIT 1 = 0 = NO WRITE VERIFY
*			    BIT 1 = 1 = WRITE VERIFY
*			    BIT 2 = 0 = FIXED HEAD AREA NOT ALLOCATED
*			    BIT 2 = 1 = FIXED HEAD AREA ALLOCATED
*					(VOLUME DIRECTORY ONLY)
*			    BIT 3 = 0 = NOT A VOLUME DIRECTORY
*			    BIT 3 = 1 = VOLUME DIRECTORY
*			    BIT 4 = 0 = DISK DIRECTORY
*			    BIT 4 = 1 = DISKETTE DIRECTORY
$$FPDIPL EQU   $$FPDFLG+2   IPL VOLUME NAME, (VOLUME DIRECTORY OR
*			    DISKETTE VOLUMES ONLY) (6)
$$FPDNUC EQU   $$FPDIPL+6   $EDXNUC SUFFIX, (VOLUME DIRECTORY OR
*			    DISKETTE VOLUMES ONLY) (1)
*	 EQU   $$FPDNUC+1   RESERVED, MUST BE ZERO (1)
$$FPDSIZ EQU   32	    LENGTH OF CONTROL ENTRY
*
***********************************
*  DIRECTORY MEMBER ENTRY  (DME)  *
***********************************
*
* THE FIRST FOUR(4) FIELDS ARE COMMON TO PROGRAMS, DATASETS, VOLUMES.
* THE REMAINING FIELDS HAVE A DIFFERENT USE - DEPENDING UPON
* WHETHER THE DME DESCRIBES A PROGRAM, DATASET, VOLUME.
*
* COMMON FIELDS:
*
$$DME	 EQU   *-*	    BEGINNING OF DME
$$FPM	 EQU   $$DME	    START FIRST FIELD
$$FPMN	 EQU   $$DME+0	    MEMBER NAME IN EBCDIC (8)
$$FPMS	 EQU   $$FPMN+8     RECORD DISPLACEMENT WITHIN DEVICE OR VOLUME
*			    TO THE FIRST RECORD OF THE MEMBER (4)
$$FPME	 EQU   $$FPMS+4     RECORD DISPLACEMENT WITHIN DEVICE OR VOLUME
*			    TO THE LAST RECORD OF THE MEMBER (4)
$$FPMT	 EQU   $$FPME+4     MEMBER TYPE INDICATOR: (2)
*			  X'0000' - UNDEFINED(RESERVED)
*			  X'0001' - DATA MEMBER
*			  X'0002' - UNDEFINED(RESERVED)
*			  X'0003' - PROGRAM MEMBER
*			  X'0004' - VOLUME(IN VOLUME DIR. ONLY)
*
$$FPMDAT EQU   X'0001'	    DATA TYPE
$$FPMPGM EQU   X'0003'	    PROGRAM TYPE
$$FPMVOL EQU   X'0004'	    VOLUME TYPE
*
* FIELDS USED FOR DATASETS AND PROGRAMS:
*
$$FPML	 EQU   $$FPMT+2     DATA MEMBER   -  RECORD LENGTH IN BYTES (2)
*			    VOLUME MEMBER -  RECORD LENGTH IN BYTES (2)
$$FPMA	 EQU   $$FPMT+2     PROGRAM MEMBER - LOAD ADDRESS (2)
***********************************
$$FPMF	 EQU   $$FPML+2     DATA MEMBER - RECORD DISPLACEMENT
*			    WITHIN THE MEMBER TO PHYSICAL RECORD
*			    CONTAINING THE NEXT AVAILABLE
*			    LOGICAL RECORD (4)
*			    VOLUME MEMBER - RESERVED, MUST BE ZERO (2)
$$FPMB	 EQU	$$FPML+2    PROGRAM MEMBER - SIZE IN BYTES (2)
***********************************
$$FPMD	 EQU	$$FPMB+4    DATA MEMBER - DISPLACEMENT IN BYTES
*			    WITHIN THE PHYSICAL RECORD FOR THE
*			    NEXT AVAILABLE LOGICAL RECORD OF THE
*			    DATA MEMBER (2)
*			    VOLUME MEMBER - RESERVED, MUST BE ZERO (2)
$$FPMP	 EQU	$$FPMB+4    PROGRAM MEMBER - ENTRY POINT ADDRESS
***********************************
$$FPMR	 EQU	$$FPMD+2    PROGRAM MEMBER - SIZE OF RELOCATION
*			    DICTIONARY(RLD)  2BYTE
*			    VOLUME MEMBER - RESERVED, MUST BE ZERO (2)
*******  EQU	$$FPMD+2    DATA MEMBER - UNUSED(RESERVED)
***********************************
*
* FLAGS AND RESERVED WORDS
*
***********************************
$$FPMFLG EQU	$$FPMR+2    FLAGS:    2BYTE
*			    BIT 0 = 1 = EOD HAS BEEN SET
*			    BIT 0 = 0 = EOD HAS NOT BEEN SET
*			    BIT 1 = RESERVED, MUST BE ZERO
*			    BIT 2 = 1 = FIXED HEAD VOLUME
*			    BIT 2 = 0 = NOT FIXED HEAD VOLUME
*******  EQU	$$FPMFLG+2  RESERVED FOR SYSTEM USE 2 BYTE
$$FPMSIZ EQU	32	    LENGTH OF DME
*
*****************************
*  FREE SPACE ENTRY  (FSE)  *
*****************************
*
$$FSE	 EQU	*-*	    BEGINNING OF FSE
$$FSEF	 EQU	$$FSE	    THE RECORD DISPLACEMENT, WITHIN THE
*			    VOLUME, TO THE FIRST RECORD OF FREE
*			    SPACE.
$$FSEL	 EQU	$$FSEF+4    THE RECORD DISPLACEMENT, WITHIN THE
*			    VOLUME, TO THE LAST RECORD OF FREE
*			    SPACE.
$$FSESIZ EQU	8	    LENGTH OF A SINGLE FSE
$$FSEFF  EQU	248	    OFFSET IN RECORD TO FIRST FSE ENTRY
***************************************************************
***************************************************************
*	 END OF DDODEFEQ
***************************************************************
***************************************************************
	 EJECT
*//
*END OF ALL OF DSOPEN
***************************************************************
***************************************************************
	 EJECT
