**********************************************************************
*								     *
*		       RENUM					     *
*								     *
*   FUNCTION							     *
*	      RENUMBER THE DATASET				     *
*								     *
*   INPUT							     *
*	      ORIGIN						     *
*		   LINE NUMBER TO BE ASSIGNED TO 'TOPLINE'	     *
*	      INCR						     *
*		   LINE NUMBER INCREMENT			     *
*	      FRSTLINE						     *
*		   CURRENT LINE NUMBER OF THE FIRST LINE TO BE	     *
*		   RENUMBERED.	IF IT IS EQUAL TO 'TOPLINE', THE     *
*		   ENTIRE DATASET IS RENUMBERED			     *
*								     *
*   OUTPUT							     *
*	      THE LINE NUMBERS IN THE INDEX RECORDS WILL HAVE BEEN   *
*	      CHANGED AND REWRITTEN ON THE DISK.		     *
*								     *
*   LOCATIONS REFERENCED					     *
*	      TOPLINE						     *
*	      DELADD+0, +2, +4					     *
*	      IPAGE						     *
*	      LINECNT						     *
*	      INCR						     *
*	      LASTLINE						     *
*	      IREWRITE						     *
*								     *
*   OTHER SUBROUTINES REFERENCED				     *
*	      FINDINDX						     *
*	      IUPDATE						     *
*								     *
*   PROCESS PERFORMED						     *
*								     *
*	      1. IF PARTIAL RENUM REQUESTED, SET STARTING LINE #     *
*								     *
*	      2. ELSE - CHECK TO ASSURE ALL LINE NUMBERS WILL BE     *
*		 VALID.  IF NOT, REDUCE INCREMENT AND ORIGIN VALUES  *
*								     *
*	      3. FIND INDEX FOR FIRST LINE (ALREADY SET IF PARTIAL   *
*		 RENUM)						     *
*								     *
*	      4. UPDATE LINE NUMBER FIELD OF INDEX ENTRY	     *
*								     *
*	      5. CHECK IF THIS IS LAST LINE OF DATA.  IF NOT,	     *
*		 FIND THE INDEX ENTRY FOR THE NEXT ONE. LOOP BACK    *
*		 TO '4'.					     *
*								     *
*	      6. AT END, RESET 'LASTLINE' TO NEW VALUE, RESTORE      *
*		 THE INDEX TO THE ORIGINAL ENTRY AND CALL 'IUPDATE'  *
*		 TO ASSURE THAT THE HEADER RECORD GETS REWRITEN.     *
*								     *
*								     *
*   CHANGE ACTIVITY:						     *
*     MAKE RENUM COPY CODE AND CORRECT RENUM WHEN  S202076 12/09/80  *
*	IN PARTIAL MODE.					     *
*								     *
**********************************************************************
	 SPACE 1
	 SUBROUT   RENUM,ORIGIN,INCR,FRSTLINE
	 SPACE 1
RENUM00  EQU	   *
	 MULTIPLY  LINECNT,INCR,1,RESULT=DELADD,PREC=SSD
	 ADD	   DELADD,ORIGIN,1,PREC=D
	 IF	   DELADD,NE,0,RENUMERR        GREATER THEN
	 IF	   DELADD+2,LT,0,RENUMERR	  32768
	 IF	   FRSTLINE,NE,TOPLINE,RENUM21 PARTIAL RENUM S2020076
	 MOVE	   TOPLINE,ORIGIN      SET NEW 'TOPLINE' NUMBER
	 SUBTRACT  DELADD+2,INCR
	 MOVE	   LASTLINE,DELADD+2   SET NEW 'LASTLINE' NUMBER
	 MOVE	   DELADD,ORIGIN
	 MOVE	   RENUM30,IPAGE
	 CALL	   FINDINDX,TOPLINE+2
	 SPACE 1
RENUM10  MOVE	   (0,#1),DELADD       STORE NEW LINE NUMBER
	 MOVE	   IREWRITE,1	       FLAG PAGE MODIFIED
	 IF	   (6,#1),EQ,0,RENUM20	CHECK FOR END OF DATA
	 ADD	   DELADD,INCR	       INCREMENT LINE NUMBER
	 MOVE	   DELADD+4,(6,#1)
	 CALL	   FINDINDX,DELADD+4
	 GOTO	   RENUM10
	 SPACE 1
RENUMERR EQU	   *
	 IF	   INCR,LE,1	       IF = 1, ERROR END PROGRAM
*	   DEQT
	   ENQT
	   PRINTEXT  '@DATA SET TOO LARGE-CANNOT RENUMBER'
	   PRINTEXT  '@ENDING $FSEDIT'
	   MOVE      #1,$PARM1					  V3M0D
*D	   MOVE      (ERRORSW,#1),-1	     FATAL ERROR  S203333 V3M0D
	   GOTO      EXIT		REQUEST PROGRAM TERMINATION
	 ENDIF
	 DIVIDE    INCR,2,RESULT=INCR  SPLIT ICREMENT AND TRY AGAIN
	 MOVE	   ORIGIN,INCR
	 GOTO	   RENUM00
	 SPACE 1
RENUM20  EQU	   *
	 MOVE  LASTLINE,DELADD	   RESET 'LASTLINE'
	 CALL	   FINDINDX,1,P2=RENUM30
	 CALL	   IUPDATE
	 RETURN
*	 MOVE THIS CODE HERE FOR FIX TO PARTIAL RENUM		S202076
RENUM21  EQU	   *
	 MOVE	   DELADD,FRSTLINE	      SET FIRST LINE NUMBER
	 MOVE	   RENUM30,IPAGE
	 GOTO	   RENUM10					S202076
	 EJECT
